from otscrape.core.base.attribute import attribute, Attribute
from otscrape.core.attribute import RequestText, RequestStatusCode, JSON, DictPath, XPath, ZipDict

from otscrape.core.io import SimpleRequestLoader, DummyLoader

from otscrape.core.base.page import Page
